#include "ESPNow.hpp"

static const char* TAG = "ESPNOW";

uint8_t BROADCAST_MAC[] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};
esp_now_peer_info_t hostInfo;
Preferences preferences;

void get_host_mac(uint8_t *destination)
{
	preferences.begin("config", true);
	if (preferences.isKey("host")) {
		preferences.getBytes("host", destination, sizeof(uint8_t) * 6);
	} else {
		memcpy(destination, BROADCAST_MAC, sizeof(BROADCAST_MAC));
		ESP_LOGW(TAG, "Backup MAC address used");
	}
	preferences.end();
}
void on_data_sent(const uint8_t *mac_addr, esp_now_send_status_t status)
{
	// go to sleep
}

void on_data_recv(const uint8_t *mac, const uint8_t *incomingData, int len)
{
	ESP_LOGI(TAG, "Message recieved");
	config new_config;
	memcpy(&new_config, incomingData, sizeof(new_config)); // TODO: check for valid mac

	// put the host address in flash mem
	preferences.begin("config", false);
	if (!preferences.isKey("host")) {
		preferences.putBytes("host", new_config.host, sizeof(new_config.host));
		ESP_LOGI(TAG, "host MAC address saved to flash");
	} // host change shouldn't be an issue
	preferences.end();
	// sync time
	esptime::rtc.setTime(
	    new_config.time_millis); // see https://www.esp32.com/viewtopic.php?t=9965, maybe this needs an offset
}

esp_err_t espnow_setup()
{
	esp_err_t result;
	WiFi.mode(WIFI_STA);
	result = esp_now_init();
	if (result != ESP_OK) {
		// initialization failed
		return result; // not sure about this
	}

	get_host_mac(hostInfo.peer_addr); // check if there is a host saved in flash mem, broadcast otherwise

	hostInfo.channel = 0;
	hostInfo.encrypt = 0;
	esp_now_add_peer(&hostInfo);

	esp_now_register_recv_cb(on_data_recv);
	esp_now_register_send_cb(on_data_sent);

	return ESP_OK;
}
